/*****************************Libraries*****************************************/
#include <Adafruit_SleepyDog.h>
#include <SoftwareSerial.h>
#include "Adafruit_FONA.h"
#include "Adafruit_MQTT.h"
#include "Adafruit_MQTT_FONA.h"
#include <LiquidCrystal.h>

/*****************************FONA Pins*****************************************/
#define FONA_RX  10
#define FONA_TX  11
#define FONA_RST 4

SoftwareSerial fonaSS = SoftwareSerial(FONA_TX, FONA_RX);

Adafruit_FONA fona = Adafruit_FONA(FONA_RST);

/*****************************APN Daten*****************************************/
#define FONA_APN       ""
#define FONA_USERNAME  ""
#define FONA_PASSWORD  ""

/*****************************Adafruit.io Setup*********************************/
#define AIO_SERVER      "io.adafruit.com"
#define AIO_SERVERPORT  1883
#define AIO_USERNAME    "USERNAME"
#define AIO_KEY         "KEY"

/*****************************MQTT Konfiguration********************************/
// Setup the FONA MQTT class by passing in the FONA class and MQTT server and login details.
Adafruit_MQTT_FONA mqtt(&fona, AIO_SERVER, AIO_SERVERPORT, AIO_USERNAME, AIO_KEY);

#define halt(s) { Serial.println(F( s )); while(1);  }

// FONAconnect is a helper function that sets up the FONA and connects to
// the GPRS network. See the fonahelper.cpp tab above for the source!
boolean FONAconnect(const __FlashStringHelper *apn, const __FlashStringHelper *username, const __FlashStringHelper *password);

/*****************************Feeds*********************************************/
// Notice MQTT paths for AIO follow the form: <username>/feeds/<feedname>
Adafruit_MQTT_Publish moisture_sensor1 = Adafruit_MQTT_Publish(&mqtt, AIO_USERNAME "/feeds/moisture-sensor-1");
Adafruit_MQTT_Publish moisture_sensor2 = Adafruit_MQTT_Publish(&mqtt, AIO_USERNAME "/feeds/moisture-sensor-2");
Adafruit_MQTT_Publish moisture_sensor3 = Adafruit_MQTT_Publish(&mqtt, AIO_USERNAME "/feeds/moisture-sensor-3");
Adafruit_MQTT_Publish moisture_sensor4 = Adafruit_MQTT_Publish(&mqtt, AIO_USERNAME "/feeds/moisture-sensor-4");
Adafruit_MQTT_Publish moisture_sensor5 = Adafruit_MQTT_Publish(&mqtt, AIO_USERNAME "/feeds/moisture-sensor-5");
Adafruit_MQTT_Publish moisture_sensor6 = Adafruit_MQTT_Publish(&mqtt, AIO_USERNAME "/feeds/moisture-sensor-6");
Adafruit_MQTT_Publish moisture_sensor7 = Adafruit_MQTT_Publish(&mqtt, AIO_USERNAME "/feeds/moisture-sensor-7");
Adafruit_MQTT_Publish moisture_sensor8 = Adafruit_MQTT_Publish(&mqtt, AIO_USERNAME "/feeds/moisture-sensor-8");
Adafruit_MQTT_Publish moisture_sensor9 = Adafruit_MQTT_Publish(&mqtt, AIO_USERNAME "/feeds/moisture-sensor-9");

/*****************************LCD Keypad Shield*********************************/
LiquidCrystal lcd(12,13,6,7,8,9);   // 12 -> RS; 13 -> E

/*****************************Variablendefinitionen*****************************/
/*Feuchtigkeitsmessung*/
uint32_t moisture[9] = {0};                        //Feuchtigkeitsmesswerte für Feld 1-9
uint32_t limit[9] = {30,30,30,30,30,30,30,30,30};  //Feuchtigkeitsgrenzwerte [%] für Feld 1-9
long long previous_time_sensor = 0;
long long interval_sensor = 60000;                 //Intervall für die Messung: 1 min
long long previous_time_water = 0;
/*Bewässerung*/
long long interval_water = 18000000;               //Intervall für die Bewässerung: 5 h
int water_counter[9] = {0};                        //Anzahl der Bewässerungen für Feld 1-9
long long water_time = 600000;                     //Bewässerungszeit: 10 min
int i = 0;
/*LCD Ausgabe*/
int lcd_power = 0;                                 //OFF: gerade; ON: ungerade
int adc_key = 0;                                   //Analoger Input-Wert der verschiedenen Buttons
int button_counter = 0;                            //Angezeigter Sensor bei bestimmtem Button-Zustand (UP: button_counter++, DOWN: button_counter--)



void setup() {
/*****************************LCD Keypad Shield*********************************/
  lcd.begin(16,2);      //16 Spalten, 2 Zeilen
  pinMode(A0, INPUT);   //Keypad Buttons
  pinMode(10, OUTPUT);  //LCD on/off
  
/*****************************Feuchtigkeitssensoren*****************************/
  /*Signale*/
  pinMode(A7, INPUT);
  pinMode(A8, INPUT);
  pinMode(A9, INPUT);
  pinMode(A10, INPUT);
  pinMode(A11, INPUT);
  pinMode(A12, INPUT);
  pinMode(A13, INPUT);
  pinMode(A14, INPUT);
  pinMode(A15, INPUT);
  
  /*Spannungsversorgung 5V*/
  pinMode(14, OUTPUT);
  pinMode(15, OUTPUT);
  pinMode(16, OUTPUT);
  pinMode(17, OUTPUT);
  pinMode(18, OUTPUT);
  pinMode(19, OUTPUT);
  pinMode(20, OUTPUT);
  pinMode(21, OUTPUT);
  pinMode(22, OUTPUT);

/*****************************Magnetventile*************************************/
  pinMode(26, OUTPUT);
  pinMode(27, OUTPUT);
  pinMode(28, OUTPUT);
  pinMode(29, OUTPUT);
  pinMode(30, OUTPUT);
  pinMode(31, OUTPUT);
  pinMode(32, OUTPUT);
  pinMode(33, OUTPUT);
  pinMode(34, OUTPUT);

/*****************************FONA Verbindung***********************************/
  //while (!Serial);
  Serial.begin(115200);
  delay(5000);  // wait a few seconds to stabilize connection
  // Initialise the FONA module
  while (! FONAconnect(F(FONA_APN), F(FONA_USERNAME), F(FONA_PASSWORD))) {
    Serial.println("Retrying FONA");
  }
  Serial.println(F("Connected to Cellular!"));
  delay(5000);  // wait a few seconds to stabilize connection
}



void loop() {
/*****************************Verbindung mit MQTT*******************************/  
  MQTT_connect();

/*****************************Messung der Feuchtigkeit**************************/
  if(millis() - previous_time_sensor > interval_sensor){
    previous_time_sensor = millis();
    while(i < 9){
      moisture[i] = measure(7+i);
      i++;
    }
    i = 0;
  
/*****************************Senden der Messwerte an Adafruit.io***************/
  moisture_sensor1.publish(moisture[0]);
  moisture_sensor2.publish(moisture[1]);
  moisture_sensor3.publish(moisture[2]);
  moisture_sensor4.publish(moisture[3]);
  moisture_sensor5.publish(moisture[4]);
  moisture_sensor6.publish(moisture[5]);
  moisture_sensor7.publish(moisture[6]);
  moisture_sensor8.publish(moisture[7]);
  moisture_sensor9.publish(moisture[8]);
  }

/*****************************Ansteuerung der Magnetventile*********************/
  while(i < 9){
    if(millis() - previous_time_water > interval_water || water_counter[i] == 0){
      if(moisture[i] < limit[i]){
        digitalWrite(26+i, HIGH);
        delay(water_time);
        digitalWrite(26+i, LOW);
        previous_time_water = millis();
        water_counter[i]++;
      }
    }
    i++;    
  }
  i = 0;

/*****************************Ausgabe der Messwerte mit LCD*********************/
  //LCD Power
  adc_key = analogRead(A0);
  if(adc_key > 450 && adc_key < 800){     //SELECT (=on/off)
    lcd_power++;
    delay(300);
  }
  if(lcd_power % 2 == 0){
    digitalWrite(10, LOW);
    digitalWrite(14+button_counter, LOW);
  }
  else{
    digitalWrite(10, HIGH); //LCD VCC  
  }

  if(lcd_power % 2 == 1){
    lcd.setCursor(0,0);
    lcd.print("Feuchtigkeit");
  
    //Überschreiten der Anzahl verfügbarer Sensoren
    if(button_counter < 0){
      button_counter = 8;
    }
    if(button_counter > 8){
      button_counter = 0;
    }
  
    //Funktion der Buttons
    if(adc_key > 150 && adc_key < 350){   //DOWN (= nächster Sensor)
      lcd.setCursor(0,1);
      lcd.print("                ");
      digitalWrite(14+button_counter, LOW);
      button_counter++;
      delay(300);
    }
    if(adc_key < 150){                    //UP (= vorheriger Sensor)
      lcd.setCursor(0,1);
      lcd.print("                ");
      digitalWrite(14+button_counter, LOW);
      button_counter--;
      delay(300);
    }
    //Ausgabe der Werte
    lcd.setCursor(0,1);
    lcd.print("Sensor ");
    lcd.print(button_counter+1);
    lcd.print(":    ");
    lcd.setCursor(10,1);
    lcd.print(moisture[button_counter]);
    lcd.setCursor(13,1);
    lcd.print("%");
    //delay(50);
    }
}



/*****************************Funktionen****************************************/
/*Messung und Berechnung der Feuchtigkeit*/
int measure(int analog_pin){
  int digital_pin = analog_pin + 7;
  digitalWrite(digital_pin, HIGH);
  delay(10);
  float sensor_value = analogRead(analog_pin);
  digitalWrite(digital_pin, LOW);
  float sensor_value_max = 570;                           //Maximaler Rückgabewert des Sensors (Wasser: 570; Kurzschluss: 690)
  int moisture = (sensor_value/sensor_value_max)*100;
  return moisture;
}

/*Verbindung mit MQTT*/
void MQTT_connect() {
  int8_t ret;
  // Stop if already connected.
  if (mqtt.connected()) {
    return;
  }
  Serial.print("Connecting to MQTT... ");
  while ((ret = mqtt.connect()) != 0) { // connect will return 0 for connected
    Serial.println("Rückgabewert mqtt.connect(): ");
    Serial.println(ret);
    delay(5000);
    Serial.println(mqtt.connectErrorString(ret));
    Serial.println("Retrying MQTT connection in 5 seconds...");
    mqtt.disconnect();
    delay(5000);  // wait 5 seconds
  }
  Serial.println("MQTT Connected!");
}
